#!/bin/bash

#*****************************************************************************#
#                               SauvegardeGeneralePC.sh                       #
#                              écrit par Rafael Venancio                      #
#                                 05 Juin 2010                                #
#                                                                             #
#*****************************************************************************#



# Versions
#==============================================================================
# 1.0 le  5/06/2010 Rafael Venancio
#
# 2.0 le 12/06/2010 
#  - ajout possibilité de multiples exclusions, de sous-répertoires (une seule exclusion auparavant).
#  - factorisation de la liste des types de fichiers excluent, permettant le réglages.
#  - prise en compte correct de toutes les exclusions dans le calcul des occupations disques. 
#  - zone des réglages initiaux plus compréhensible
#  - simplification du code 


# Ce que fait le script
#==============================================================================
# - Réalise la sauvegarde des repertoires et partitions sélectionnés d'un PC,
#   sur un disque dure USB externe, entièrement automatiquement. Sauvegarde
#   totale, pas incrémentale.
# - Crée des archives (non compressés) en découpant celle-ci en fichiers de 2 Go
#   maximum chacun, avec l'application 'dar' http://dar.lunix.free.fr ,
#   http://fr.wikipedia.org/wiki/DAR (en français).	
# - Vérifie que l'espace libre sur le disque du PC permet la création locale des 
#   fichiers de 2 Go.
# - Vérifie que le disque dure USB externe dispose de suffisemment d'espace libre
#   pour la sauvegarde totale. 
# - Crée un répertoire de sauvegarde daté, sur le disque USB. 
# - Calcul un checksum sha1, http://fr.wikipedia.org/wiki/SHA-1, pour chaque 
#   fichier d'archive.
# - Copie les fichiers archives et les checksum de fichiers sur le disque USB.
# - Vérifie le checksum des fichiers une fois ceux-ci sur le disque USB (copie 
#   non corrompue).
# - supprime les fichiers d'archive du PC une fois réalisé la vérification.
# - Effectue toutes ces opérations sur chaque fichier d'archive de 2 Go, au fur 
#   et à mesure, puis efface celui-ci du PC avant de créer le suivant. Pour 
#   réduire l'espace disque nécessaire sur le PC au fonctionnent du script.
# - Crée un fichier des checksum de l'ensemble d'une archive pour une vérification
#   manuelle total, si necessaire.
# - Crée un fichier de Log, équivalent à l'affichage sur la console, et copie 
#   celui-ci sur le disque USB.
# - Copie les fichiers du script sur le disque USB.

# Utilisation des sauvegardes
#==============================================================================
# - Desarchivage des sauvegardes globales ou fichier individuel avec 
#   l'application 'DarGUI' pour Linux, http://sourceforge.net/projects/dargui/.
# - Il est possible d'utiliser 'dar' en ligne de commande mais je ne vois pas
#   l'interrêt. 

# Mode d'emploi au quotidien
#==============================================================================
# - Brancher le disque dure USB externe et vérifier qu'il est accessible.
# - Stopper les programmes en cours qui pourrais modifier les données à sauvegarder.
# - Ouvrir une console dans le répertoire du PC qui contient ce script.
# - Taper la commande 'sh SauvegardeGeneralePC.sh' et faire Entrée.
# - Pendant la sauvegarde, ne pas faire fonctionner des programmes qui pourraient
#   modifier les données à sauvegarder.

# Commentaires et remarques
#==============================================================================
# - Au cours de la sauvegarde les fichiers d'archives sont créer dans le répertoire
#   ou se trouve le script. 
# - Les fichiers iso (images de CD/DVD) ne sont pas archivées car je considère
#   que ces données sont toujours téléchargeables. Ces exclusions sont réglables.
# - La durée de sauvegarde est plus influencée par la puissance du processeur que
#   le volume des données. Ceci en raison de la création des archives et des
#   calculs des cheksum sha1. 65 Go de sauvegarde prennent 2h30 sur mon AMD Athlon64
#   et environ 7h00 sur mon NetBook MSI.

# Installation et paramétrage à faire la 1ère fois.
#==============================================================================
# - Utiliser de préference un éditeur de fichier type Gedit ou Kate. La coloration
#   syntaxique de ces éditeurs facilitera les réglages.
# - Vérifier que les requis ci-dessous sont respectés.
# - Copier ce fichier et son sous-programme 'SauvegardeGeneralePc_SousProgr1.sh'
#   dans un répertoire du PC avec au moins 5 Go d'espace disque libre. 
# - Attention, le répertoire dans lequel est placé le script n'est pas sauvegardé,
#   ne pas placer de données dans ce répertoire.
# - Régler l'arborescence de destination vers le disque USB externe.
#   Voir '>>>>> REGLAGE INITIAL 1 <<<<<' dans le code ci-dessous.
#   Cette arboresence doit correspondre à celle existante jusqu'au repertoire 
#   de destination sur le disque USB. Elle n'est pas crée par le script. 
# - Régler les partitions et repertoires à sauvegarder et à exclure dans la 
#   fonction 'Partitions et répertoires de sauvegarde'.
#   Voir '>>>>> REGLAGE INITIAL 2 <<<<<' dans le code ci-dessous.
# - Régler éventuellement les types de fichiers qui seront excluent.
#   Voir '>>>>> REGLAGE INITIAL 3 <<<<<' dans le code ci-dessous. 
# - Lancer le script jusqu'à la vérification de l'espaces disques disponibles 
#   et necessaires. Ceci permet de vérifier l'arborescence des répertoires à 
#   sauvegarder. 
# - Conseil : utiliser un logiciel de visualisation l'occupation disque du PC
#   sous forme graphique (exemple : xDiskUsage sous Linux) pour décider ce qu'il 
#   est necessaire de sauvegarder.

# Requis
#==============================================================================
# - Paquet 'dar'. Pour vérifier sa présence, taper en console : "dar --help"
# - Paquet 'bc'. Programme à part, qui fournit un langage de calcul. Pour 
#   vérifier sa présence, taper en console : "bc --help"
# - 'sha1sum'. Il est souvent installé par defaut. Pour vérifier sa présence, 
#   taper en console : "sha1sum --help"
# - Le script de sous-programme 'SauvegardeGeneralePc_SousProgr1.sh' dans le même
#   repertoire de le script principal. 
# - 'DarGui' pour relire les archives en mode graphique, les extraires en totalité
#   ou un seul fichier. 

# Principe de l'arborescence des sauvegardes sur le disque dure USB
#==============================================================================
# Voici l'arborescence que j'utilise personnellement.
#
# SAUVEGARDE (nom du disque dure USB)
#     |
#      --- AmdAthlon64 (un de mes PC)
#     | 
#      --- Portable_1 (un de mes PC)
#     | 
#      --- PowerNet (un de mes PC)
#     | 
#      --- Netbook_MSI (le PC de ce script) <<<<< REGLAGE INITIAL 1
#            |
#             --- Sauvg_2010-05-01 (création automatique)
#            |
#             --- Sauvg_2010-05-15 (création automatique)
#            |
#             --- Sauvg_2010-06-01 (création automatique)
#                    |
#                     --- Mandriva (partition Linux) <<<<< REGLAGE INITIAL 2 - Partition PC
#                    |       |
#                    |        --- papa (mon /home) <<<<< REGLAGE INITIAL 2 - Répertoire PC
#                    |       |
#                    |        --- thunderbird (mes courriels) <<<<< REGLAGE INITIAL 2 - Répertoire PC
#                    |
#                     --- WindowsXP (partition Windows) <<<<< REGLAGE INITIAL 2 - Partition PC
#                    |       |
#                    |        --- Bureau (le bureau) <<<<< REGLAGE INITIAL 2 - Répertoire PC
#                    |       |
#                    |        --- MesDocuments (mes documents) <<<<< REGLAGE INITIAL 2 - Répertoire PC
#                    | 
#                     --- Disque (partition de partage Linux/Windows) <<<<< REGLAGE INITIAL 2 - Partition PC
#
# Pourquoi cette arborescence ?
# - j'ai plusieurs PC.
# - je sauvegarde tous mes PC sur un même disque dure USB externe.
# - je sauvegarde l'intégralité du PC à chaque fois.
#
# Ceci me permet :
# - de facilement retrouver les sauvegardes par rapport aux dates.
# - de facilement effacer les plus anciennes sauvegarde (répertoire 'Sauvg_AAAA-MM-JJ')
#   pour n'en conserver que 3. 
# - si je souhaite placer une sauvegarde sur un site en ligne, je prend une 
#   sauvegarde à date.

# Tests avec une clef USB au lieu d'un disque USB
#==============================================================================
# - Régler la fonction'Partitions et répertoires de sauvegarde pour test sur une
#   clef USB' avec un répertoire de destination vers une clef USB. 
# - Taper la commande 'sh SauvegardeGeneralePC.sh test'. Le mot 'test' est ajouté. 

# Les raisons de mes choix
#==============================================================================
# - Création d'archives au lieu de sauvegardes directes des fichiers : 
#   Pas de soucis de copie sur le disque USB même avec des arborescences très 
#   complexes des partitions et répertoires.
#   La création d'archive de 2 Go réduit très fortement le nombre de fichier à 
#   copier.
#   Permet de cacluler facilement des checksums car peu de fichiers.
# - Calcul de checksum 'sha1' : 
#   Permet de vérifier que les archives ne sont pas corrompues au file du temps,
#   avec un stockage de longue durée ou lors des copies d'un support à l'autre
#  (disque dure, disque USB, etc..).
# - Pourquoi ne pas compresser les archives ? :
#   Le plus grand volume de mes données sont des photos en jpg, qui sont donc 
#   déjà compressées. Inutile de les recompresser.
#   Les capacités/prix des disques dures USB externes d'aujourd'hui réduit
#   formtement la necessité de compression des données. 
#   La compression de fichiers augmente énormément le temps pour créer l'archive,
#   il faut donc le faire lorsque c'est vraiment necessaire.   

# A faire
#==============================================================================
#
# Pour une prochaine version : 
# - Vérifier s'il y a plus de 3 (réglable) sauvegardes valide sur le disque USB. Si c'est le cas, effacer les plus anciennes.
# - exclure de base le répertoire locale, ce qui simplifie l'exclusion.
# - placer un test dans sous-progre pour qu'il ne fct qu'avec ce script. Si aucun paramètre n'est passé à la fonction : "if [ "$#" = 0 ]; then ..."
# - sauvegarder les fichiers et dossiers de configurations du /home ?
# - vérifier les requis au demarrage du script. 
# - Créer une fonction 'interruption'.
# - En cas d'interruption, effacer le repertoire de sauvegarde si aucun archive n'a encore été crée.
# - Au lancement, vérifier la dernière sauvegarde (fichier log). Si interruption, information ? Action ? 
# - ? Créer un répertoire temporaire pour la création temporaire des archives, afin d'éviter l'exclusion du répertoire 'dar_backups




# >>>>> REGLAGE INITIAL 1 - DEBUT <<<<<
# Repertoire de destination sur le disque dure USB externe
#*******************************************************************************
# Ce repertoire doit correspondre à un PC (Netbook, AmdAthlon64, Portable_1, etc...)
# ATTENTION : pas d'espace dans le nom du répertoire, utiliser le tiret milieu '-'
#             ou tiret bas '_'.

if [ "$1" != "test" ] ; then
	targetDir="/media/SAUVEGARDE/Netbook_MSI" # <<<<< REGLAGE INITIAL
else
    targetDir="/media/CLEFUSB1/Netbook_MSI" # uniquement pour tests sur clef USB
fi
# >>>>> REGLAGE INITIAL 1 - FIN <<<<<



# Taille des fichiers d'archives (optionnel)
#*******************************************************************************
if [ "$1" != "test" ] ; then
	tailleFichierArchive="2000" # Mo (2000 = 2000 Mo = 2 Go, 3000 = 3 Go)
else
	tailleFichierArchive="100" # Mo - Pour tests sur clef USB (100 = 100 Mo = 0,1 Go)
fi




################################################################################
################################################################################
#                                  FONCTIONS 
################################################################################
################################################################################



# >>>>> REGLAGE INITIAL 2 - DEBUT <<<<<
#*******************************************************************************
# fonction - Partitions et répertoires de sauvegarde
#*******************************************************************************
# Commentaires pour les réglages :
# - Pour créer d'autre partition, copier tout un bloc existant puis modifier. Supprimer ce qui n'est pas necessaire.
# - Ne pas utiliser d'espace dans les noms qui sont créer, utiliser des '_' ou'-'. 
# - Respecter la syntaxe des chemins de répertoire sur la présence ou absence de '/'.
# - Le chemin du répertoire du PC à archiver doit respecter l'arborescence et les noms réelles.
# - Dé-commenter le début de ligne (supprimer le '#') pour créer des exclusions.
# - Les exclusions doivent respecter les noms réelles des sous-répertoires.

archiver()
{
	# PARTITION PC 1 
	#----------------------------------------------------------------------------
	nomPartitionPC="Mandriva"                                      # <<<<< REGLAGE : nom de la partition PC.
	afficherTitrePartitionEtAutre # fonction
	#----------------------------------------------------------------------------
	repertoire_a_archiver="/home/papa/"                            # <<<<< REGLAGE : chemin du répertoire du PC à archiver.
	nomArchiveDar="papa"                                           # <<<<< REGLAGE : nom de l'archive sur disque USB, pour ce répertoire.
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers                # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure. Un par ligne. 
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers 	       # <<<<< REGLAGE : nom sous-sous-repertoire PC (ZZZZZ) à exclure. Un par ligne.
	sauvegarder # fonction  
	#----------------------------------------------------------------------------
	repertoire_a_archiver="/home/papa/.thunderbird/"               # <<<<< REGLAGE : chemin du répertoire du PC à archiver
	nomArchiveDar="thunderbird"                                    # <<<<< REGLAGE : nom de l'archive sur disque USB, pour ce répertoire.
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers                # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure, un par ligne 
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers 	       # <<<<< REGLAGE : nom sous-sous-repertoire PC (ZZZZZ) à exclure. Un par ligne.
	sauvegarder # fonction  
	#----------------------------------------------------------------------------
	


	# PARTITION PC 2 
	#----------------------------------------------------------------------------
	nomPartitionPC="WindowsXP"                                     # <<<<< REGLAGE : nom de la partition PC.
	afficherTitrePartitionEtAutre # fonction
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/mnt/windows/Documents and Settings/Rafael Venancio/Bureau/" # <<<<< REGLAGE : chemin du répertoire du PC à archiver.
	nomArchiveDar="Bureau"                                         # <<<<< REGLAGE : nom de l'archive sur disque USB, pour ce répertoire.
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers                # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure. Un par ligne.
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers 	       # <<<<< REGLAGE : nom sous-sous-repertoire PC (ZZZZZ) à exclure. Un par ligne.
	sauvegarder # fonction  
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/mnt/windows/Documents and Settings/Rafael Venancio/Mes documents/" # <<<<< REGLAGE : chemin du répertoire du PC à archiver.
	nomArchiveDar="MesDocuments"                                   # <<<<< REGLAGE : nom de l'archive sur disque USB, pour ce répertoire.
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers                # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure. Un par ligne. 
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers 	       # <<<<< REGLAGE : nom sous-sous-repertoire PC (ZZZZZ) à exclure. Un par ligne.
	sauvegarder # fonction
	#----------------------------------------------------------------------------  



	# PARTITION PC 3 
	#----------------------------------------------------------------------------
	nomPartitionPC="Disque"                                         # <<<<< REGLAGE : nom de la partition PC.
	afficherTitrePartitionEtAutre # fonction
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/mnt/Disque/"                            # <<<<< REGLAGE : chemin du répertoire du PC à archiver.
	nomArchiveDar="Disque"                                          # <<<<< REGLAGE : nom de l'archive sur disque USB, pour ce répertoire.
	echo "Video" >> exclusionRepertoiresEtFichiers                  # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure. Un par ligne.
	echo "Images" >> exclusionRepertoiresEtFichiers                 # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure. Un par ligne.
	echo "Musique" >> exclusionRepertoiresEtFichiers                # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure. Un par ligne.
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers                 # <<<<< REGLAGE : nom sous-repertoire PC (XXXXX) à exclure. Un par ligne.
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers 	        # <<<<< REGLAGE : nom sous-sous-repertoire PC (ZZZZZ) à exclure. Un par ligne.
	sauvegarder # fonction
	#----------------------------------------------------------------------------  
}
# >>>>> REGLAGE INITIAL 2 - FIN <<<<<




# >>>>> REGLAGE INITIAL 3 - DEBUT <<<<<
#*******************************************************************************
# fonction - Complétion de la liste des type de fichiers (extensions) à exclure
#*******************************************************************************
typeFichiers_a_exclure()
{
	# Pour ajouter d'autres exclusions, ajouter des lignes en respectant la syntaxe
	echo "*.*~" >> exclusionTypesFichier
	echo "*~" >> exclusionTypesFichier
	echo "*.dar" >> exclusionTypesFichier
	echo "*.iso" >> exclusionTypesFichier
}
# >>>>> REGLAGE INITIAL 3 - FIN <<<<<



#*******************************************************************************
# fonction - Partitions et répertoires de sauvegarde pour test sur
#            un clef USB au lieu d'un disque USB
#*******************************************************************************
archiverTestSurClefUsb()
{
	# PARTITION PC 1 
	#-------------------------------------------------------------------------------
	nomPartitionPC="Mandriva"
	afficherTitrePartitionEtAutre # fonction
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/home/papa/rpm/"
	nomArchiveDar="rpm"
	echo "BUILD" >> exclusionRepertoiresEtFichiers
	echo "SOURCES" >> exclusionRepertoiresEtFichiers
	echo "RPMS/i586" >> exclusionRepertoiresEtFichiers
	sauvegarder $1 $2 # fonction
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/home/papa/.thunderbird/"
	nomArchiveDar="thunderbird"
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers  
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers 
	sauvegarder $1 $2 # fonction
	#-------------------------------------------------------------------------------



	# PARTITION PC 2
	#-------------------------------------------------------------------------------
	nomPartitionPC="WindowsXP"
	afficherTitrePartitionEtAutre # fonction
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/mnt/windows/Documents and Settings/Rafael Venancio/Bureau/"
	nomArchiveDar="Bureau"
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers 
	sauvegarder $1 $2 # fonction
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/mnt/windows/Documents and Settings/Rafael Venancio/Mes documents/"
	nomArchiveDar="MesDocuments"
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers
	sauvegarder $1 $2 # fonction
	#-------------------------------------------------------------------------------



	# PARTITION PC 3
	#-------------------------------------------------------------------------------	
	nomPartitionPC="Disque"
	afficherTitrePartitionEtAutre # fonction
	#-------------------------------------------------------------------------------
	repertoire_a_archiver="/mnt/Disque/Documents/Wind U100/"
	nomArchiveDar="Wind_U100"
	echo "Fond ecran" >> exclusionRepertoiresEtFichiers
	#echo "XXXXX" >> exclusionRepertoiresEtFichiers
	#echo "YYYYY/ZZZZZ" >> exclusionRepertoiresEtFichiers
	sauvegarder $1 $2 # fonction
	#-------------------------------------------------------------------------------
}




#*******************************************************************************
# fonction - complétion du fichier de commande pour dar
#*******************************************************************************
creerCommandeDar()
{
	echo "-c $nomArchiveDar" >> commandeDar
	echo "-R \"$repertoire_a_archiver\"" >> commandeDar
	echo "-w" >> commandeDar
	echo "-s $tailleFichierArchive" >> commandeDar
	echo "-D" >> commandeDar
	echo "--exclude-from-file exclusionRepertoiresEtFichiers" >> commandeDar
	echo "-E \"./SauvegardeGeneralePC_SousProgr1.sh %b %n $formatFichier $targetDir $backupDir $nomPartitionPC\"" >> commandeDar

	# exclusion des type de fichiers
	while read line
	do
		echo -e "-X \"$line\" " >> commandeDar
	done < exclusionTypesFichier
}




#*******************************************************************************
# fonction - Création de sous-répertoire sur le disque USB 
#*******************************************************************************
creerSousRepertoireSurUSB()
{
	echo -n "Création sous-répertoire '$nomPartitionPC' sur disque USB... " | tee -a $fichierLog

	# Vérifier si le répertoire à créer sur le disque USB exite déjà
	if [ -d "$targetDir/$backupDir/$nomPartitionPC" ]; then 
		echo " " | tee -a $fichierLog
		echo "Le répertoire '$nomPartitionPC' existe déjà sur le disque USB" | tee -a $fichierLog
		read -p "    -> Poursuivre en ajoutant les fichiers dans ce répertoire ? [o]ui  [n]on " ERR1
		echo "    -> Poursuivre en ajoutant les fichiers dans ce répertoire ? [o]ui  [n]on : $ERR1" >> $fichierLog
		if [ "$ERR1" != "o" ] ; then
			echo " " | tee -a $fichierLog
			echo "Interruption du script." | tee -a $fichierLog
			echo "Aucune action réalisée. Corriger manuellement et relancer le script" | tee -a $fichierLog
			echo " " | tee -a $fichierLog
			echo " " | tee -a $fichierLog
			echo " " | tee -a $fichierLog
			setterm -cursor on # Demasquage du curseur
			exit 0
		fi
	else
		mkdir "$targetDir/$backupDir/$nomPartitionPC"
		if [ "$?" == "0" ]; then # le '$?' == code sortie de la dernière commande'
			echo "OK" | tee -a $fichierLog
		else
			echo "echec" | tee -a $fichierLog
			echo " " | tee -a $fichierLog
			echo "Interrupt	ion du script." | tee -a $fichierLog
			echo "Vérifier la cause de la non création du répertoire" | tee -a $fichierLog 
			echo " " | tee -a $fichierLog
			echo " " | tee -a $fichierLog
			echo " " | tee -a $fichierLog
			setterm -cursor on # Demasquage du curseur
			exit 1

		fi
	fi
}




#*******************************************************************************
# fonction - Affichage titres des partitions et autres actions
#*******************************************************************************
afficherTitrePartitionEtAutre()
{
	verifierLongeurNomPartition # fonction

	if [ "$verificationEspaceDisque" != "o" ] ; then
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		echo "************************************************************************" | tee -a $fichierLog
		echo -n "                 partition '$nomPartitionPC'" | tee -a $fichierLog

		if [ "$test" == "o" ] ; then
			echo " -> clef USB" | tee -a $fichierLog
		else
			echo " " | tee -a $fichierLog
		fi

		echo "************************************************************************" | tee -a $fichierLog

		creerSousRepertoireSurUSB # Fonction
	fi
}



#*******************************************************************************
# fonction - Sauvegarde 
#*******************************************************************************
sauvegarder()
{
	verifierLongeurNomArchiveDar # fonction

	> exclusionTypesFichier # Création fichier des type de fichier (*.mp3, *.iso, etc..) à exclure
	typeFichiers_a_exclure # fonction - complétion du fichier des type de fichiers à exclure

	if [ "$verificationEspaceDisque" == "o" ] ; then
		echo -n "- partition " | tee -a $fichierLog
		printf "%-"$longueurMaxiNomPartition"s %s" "'$nomPartitionPC'" | tee -a $fichierLog
		echo -n " repert. " | tee -a $fichierLog
		printf "%-"$longueurMaxiNomArchiveDar"s %s" "'$nomArchiveDar'" | tee -a $fichierLog
		echo -n "... " | tee -a $fichierLog

		# Occupation disque
		occupationDisqueRepertoirePc=`du -s -k --exclude-from=exclusionRepertoiresEtFichiers --exclude-from=exclusionTypesFichier "$repertoire_a_archiver" | awk '{print $1}'`

		occupationDisqueRepertoirePcTotal=`expr $occupationDisqueRepertoirePcTotal + $occupationDisqueRepertoirePc`

		afficherTaille $occupationDisqueRepertoirePc 2 # fonction
	
	else
		echo " "  | tee -a $fichierLog
		echo "Archive : $nomArchiveDar" | tee -a $fichierLog
		echo "**************************************************************************" | tee -a $fichierLog

		# archivage
		> commandeDar # création fichier de commande pour 'dar'
		creerCommandeDar # fonction - complétion du fichier de commande pour 'dar'

		dar -B commandeDar | tee -a $fichierLog

		rm commandeDar

		creerFichierSha1Archive # fonction

	fi

	rm exclusionTypesFichier
}




#*******************************************************************************
# fonction - Création du fichier sha1 de l'ensemble des fichiers d'une archive
#            Pour pouvoir vérifier (sha1) tous les fichiers d'une archive en une fois, manuellement
#*******************************************************************************
creerFichierSha1Archive()
{
	# Création d'un fichier avec les nom des différents fichiers sha1 d'une archive
	find "$targetDir/$backupDir/$nomPartitionPC/" -name "$nomArchiveDar.*.sha1" -type f > fichiersSurPC
	exec 3<fichiersSurPC # Stockage du fichier dans un buffer

	# Création d'un fichier avec les codes et noms, formaté pour le 'sha1sum -c ....'
	while read ligneFichiersSurPC 0<&3
	do
		fichierPC=${ligneFichiersSurPC##*\/} # extraction du nom de fichier, sans l'arboresence

		# lecture les mots 1 et 2 
		while IFS=: read codeSha1 nomFichier 
		do
			echo -e "$codeSha1 $nomFichier" >> "tempo"
		done < "$targetDir/$backupDir/$nomPartitionPC/$fichierPC"
	done 

	# Suppression des espaces en fin de ligne (sinon la cde 'sha1sum -c ...' ne fonctionne pas)
	sed 's/ *$//g' "tempo" > "$targetDir/$backupDir/$nomPartitionPC/$nomArchiveDar.sha1" 

	rm fichiersSurPC
	rm tempo
}




#*******************************************************************************
# fonction - afficher automatiquement l'unité d'espace disque la plus 
#            pertinente : kb, Mb, Gb, Tb
#*******************************************************************************
afficherTaille()
{
	# $1 = valeur à afficher
	# $2 = nombre de décimale

	if [ "$1" -gt "1000000000" ] ; then
		valeur=$(echo "scale=$2; $1/1000000000" | bc -l)
		printf "%7s" $valeur | tee -a $fichierLog
		echo " To" | tee -a $fichierLog
	else
		if [ "$1" -gt "1000000" ] ; then
			valeur=$(echo "scale=$2; $1/1000000" | bc -l)
			printf "%7s" $valeur | tee -a $fichierLog
			echo " Go" | tee -a $fichierLog
		else
			if [ "$1" -gt "1000" ] ; then
				valeur=$(echo "scale=$2; $1/1000" | bc -l)
				printf "%7s" $valeur | tee -a $fichierLog
				echo " Mo" | tee -a $fichierLog
			else
				valeur=$(echo "scale=$2; $1/1" | bc -l)
				printf "%7s" $valeur | tee -a $fichierLog
				echo " Ko" | tee -a $fichierLog
			fi
		fi
	fi
}




#*******************************************************************************
# fonction - vérifier la longueur du nom de partition '$nomPartitionPC'
#*******************************************************************************
verifierLongeurNomPartition()
{
	longueurNomdiskDir=`expr length $nomPartitionPC`
	valeur=`expr $longueurMaxiNomPartition - 2` # Correction de 2 semble necessaire. 

	if [ $longueurNomdiskDir -gt $valeur ]; then
		echo " " | tee -a $fichierLog		
		echo "Interruption du script." | tee -a $fichierLog
		echo "- Le nom de partition '$nomPartitionPC' est trop long ($longueurNomdiskDir)." | tee -a $fichierLog 
		echo "- Maximum autorisé : $valeur caractères." | tee -a $fichierLog 
		echo "- Racourcir le non de la partition." | tee -a $fichierLog 
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		setterm -cursor on # Demasquage du curseur
		exit 1
	fi
}




#*******************************************************************************
# fonction - vérifier la longueur du nom de l'archive '$nomArchiveDar'
#*******************************************************************************
verifierLongeurNomArchiveDar()
{
	longueurNomArchiveDar=`expr length $nomArchiveDar`
	valeur=`expr $longueurMaxiNomArchiveDar - 2` # Correction de 2 semble necessaire. 

	if [ $longueurNomArchiveDar -gt $valeur ]; then
		echo " " | tee -a $fichierLog		
		echo "Interruption du script." | tee -a $fichierLog
		echo "- Le nom de l'archive '$nomArchiveDar' est trop long ($longueurNomArchiveDar)." | tee -a $fichierLog 
		echo "- Maximum autorisé : $valeur caractères." | tee -a $fichierLog 
		echo "- Racourcir le non de l'archive." | tee -a $fichierLog 
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		setterm -cursor on # Demasquage du curseur
		exit 1
	fi
}







################################################################################
################################################################################
#                                 FLUX PRINCIPAL
################################################################################
################################################################################

# Variable _____________________________________________________________________
formatFichier="dar" # Extension du fichier Dar
DATE=`date -I` # Date au format AAAA-MM-JJ
DATEHEURE="`date -I`_`date +%H%M%S`"

# Fichiers pour exclutions pour les cde 'dar' et 'du' (disk usage)
> exclusionRepertoiresEtFichiers # Noms (désignation) des répertoires et fichiers


# Réglages _____________________________________________________________________
longueurMaxiNomPartition=16 # Longueur maximum du nom de la partition
longueurMaxiNomArchiveDar=25 # Longueur maximum du non de l'arcvive.
# Ces longueurs de nom sont utilisées pour la mise en page de l'affichage des 
# occupation disques PC des archives.



fichierLog="./sauvegardeGeneralePC_$DATEHEURE.log" # Création du fichier de log


echo "************************************************************************" | tee -a $fichierLog
echo "*  Script d'archivage et copie des fichiers sur un disque USB externe  *" | tee -a $fichierLog 
echo "*                                                                      *" | tee -a $fichierLog 
echo "*               Rafael Venancio rafael.venan@free.fr                   *" | tee -a $fichierLog 
echo "************************************************************************" | tee -a $fichierLog 




#*******************************************************************************
# Vérification que l'espace disque libre du PC est suffisant pour creer les 
# fichiers d'archives
#*******************************************************************************
echo " " | tee -a $fichierLog
echo -n "Vérification espace disponible PC pour création fichiers archives... " | tee -a $fichierLog

espaceDisponibleDisquePc=`df -k "./" | grep % | awk '{print $4}'` # '$4' représente le 4ème élement de la ligne 'df -k'
espaceDisponibleDisquePcMini=`expr $tailleFichierArchive \\* 2 \\* 1000` # coefficient 2 et conversion en Kb


if [ $espaceDisponibleDisquePcMini -gt $espaceDisponibleDisquePc ]; then
	echo "échec" | tee -a $fichierLog

	echo -n "- Espace disponible sur disque PC                                 : " | tee -a $fichierLog
	afficherTaille $espaceDisponibleDisquePc 2 # fonction

	echo -n "- Taille des fichiers archive                                     : " | tee -a $fichierLog
	afficherTaille `expr $tailleFichierArchive \\* 1000` 2 # fonction et conversion en Kb

	espaceDisquePcManquant=`expr $espaceDisponibleDisquePcMini - $espaceDisponibleDisquePc`

	echo -n "- Il manque                                                       : " | tee -a $fichierLog
	afficherTaille $espaceDisquePcManquant 2 # fonction

	echo " " | tee -a $fichierLog
	echo "Interruption du script." | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	setterm -cursor on # Demasquage du curseur
	exit 0	
fi

echo "OK" | tee -a $fichierLog

echo -n "- Espace disponible sur disque PC                                 : " | tee -a $fichierLog
afficherTaille $espaceDisponibleDisquePc 2 # fonction




#*******************************************************************************
# Confirmation branchement et reconnaissance du disque USB
#*******************************************************************************
echo " " | tee -a $fichierLog
echo "Connecter et allumer le disque dur USB externe" | tee -a $fichierLog
echo "Attention ! attendre qu'il soit reconnu par le système avant de poursuivre" | tee -a $fichierLog

read -p "    -> confirmation disque dur USB disponible ? [o]ui  [n]on : " ERR3
echo "    -> confirmation disque dur USB disponible ? [o]ui  [n]on : $ERR3" >> $fichierLog

if [ "$ERR3" != "o" ] ; then
	echo " " | tee -a $fichierLog
	echo "Interruption du script." | tee -a $fichierLog
	echo "Aucune action réalisée." | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	setterm -cursor on # Demasquage du curseur
	exit 0
fi




#*******************************************************************************
# Vérification présence arboresence pour ce PC sur le disque USB
#*******************************************************************************
echo " " | tee -a $fichierLog
echo "Vérification arboresence pour ce PC sur le disque USB" | tee -a $fichierLog

echo -n "'$targetDir'... " | tee -a $fichierLog

if [ -d "$targetDir" ]; then
	echo "OK"  | tee -a $fichierLog
else
	echo "echec" | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo "Interruption du script." | tee -a $fichierLog
	echo "Modifier l'arboresence définie dans ce script ou créer l'arborescence"  | tee -a $fichierLog
	echo "ci-dessus sur le disque USB." | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	setterm -cursor on # Demasquage du curseur
	exit 1
fi




#*******************************************************************************
# Calcul occupation disque de la sauvegarde
#*******************************************************************************
setterm -cursor off # Masquage du curseur

echo " " | tee -a $fichierLog
echo "Calcul occupation disque de la sauvegarde" | tee -a $fichierLog


# Espace minimum autorisé sur les disque USB après la sauvegarde _______________
espaceDisqueDisponibleFinaleMiniAdm=`expr $tailleFichierArchive \\* 3 \\* 1000` # coefficient 3 et conversion en kb


# Calcul des occupation disque des répertoires à sauvegarder ___________________
occupationDisqueRepertoirePcTotal="0"

verificationEspaceDisque="o" # pour réaliser le calcul d'espace disque au lieu d'un archivage
if [ "$1" != "test" ] ; then
	archiver # fonction
else
	archiverTestSurClefUsb $1 $2 # fonction 'archiver' pour tests avec clef USB
fi
verificationEspaceDisque="n" # pour les prochains archivages soient réels

echo -n "  Prévision occupation disque de la sauvegarde                total " | tee -a $fichierLog
afficherTaille $occupationDisqueRepertoirePcTotal 2 # fonction




#*******************************************************************************
# Vérification espace disponible suffisant sur le disque USB
#*******************************************************************************
echo " " | tee -a $fichierLog
echo -n "Verification espace disponible disque USB... " | tee -a $fichierLog

espaceDisponibleDisqueUsb=`df -k "$targetDir" | grep % | awk '{print $4}'` # '$4' représente le 4ème élement de la ligne 'df -k'
occupationDisqueUsb=`du -s -k "$targetDir" | awk '{print $1}'`

capaciteMaxiDisqueUsb=`expr $espaceDisponibleDisqueUsb + $occupationDisqueUsb`

previsionEspaceTotalDisqueUsbFinale=`expr $occupationDisqueUsb + $occupationDisqueRepertoirePcTotal + $espaceDisqueDisponibleFinaleMiniAdm`

previsionEspaceDisponibleDisqueUsbFinale=`expr $espaceDisponibleDisqueUsb - $occupationDisqueRepertoirePcTotal`

if [ $previsionEspaceTotalDisqueUsbFinale -gt $capaciteMaxiDisqueUsb ]; then
	echo "échec" | tee -a $fichierLog

	echo -n "- Espace disponible sur disque USB                                : " | tee -a $fichierLog
	afficherTaille $espaceDisponibleDisqueUsb 2 # fonction

	espaceDisqueUsbManquant=`expr $occupationDisqueRepertoirePcTotal - $espaceDisponibleDisqueUsb + $espaceDisqueDisponibleFinaleMiniAdm`

	echo -n "- Il manque                                                       : " | tee -a $fichierLog
	afficherTaille $espaceDisqueUsbManquant 2 # fonction

	echo " " | tee -a $fichierLog
	echo "Interruption du script." | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	setterm -cursor on # Demasquage du curseur
	exit 0	
fi

echo "OK" | tee -a $fichierLog

echo -n "- Espace disponible sur disque USB                                : " | tee -a $fichierLog
afficherTaille $espaceDisponibleDisqueUsb 2 # fonction

echo -n "- Prevision espace disponible sur disque USB après la sauvegarde  : " | tee -a $fichierLog
afficherTaille $previsionEspaceDisponibleDisqueUsbFinale 2 # fonction

setterm -cursor on # Demasquage du curseur

read -p "    -> Lancer la sauvegarde ? [o]ui  [n]on : " ERR1
echo "    -> Lancer la sauvegarde ? [o]ui  [n]on : $ERR1" >> $fichierLog
if [ "$ERR1" != "o" ] ; then
	echo " " | tee -a $fichierLog
	echo "Interruption du script." | tee -a $fichierLog
	echo "Aucune action réalisée." | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	echo " " | tee -a $fichierLog
	setterm -cursor on # Demasquage du curseur
	exit 0
fi




#*******************************************************************************
# Création répertoire de sauvegarde sur le disque USB
#*******************************************************************************
backupDir="Sauvg_$DATE"
echo " " | tee -a $fichierLog
echo -n "Création répertoire '$backupDir' sur disque USB... " | tee -a $fichierLog

if [ -d "$targetDir/$backupDir" ]; then # Vérifier si le répertoire à créer sur le disque USB exite déjà
	echo " "  | tee -a $fichierLog
	echo "Le répertoire '$backupDir' existe déjà sur le disque USB" | tee -a $fichierLog
	read -p "    -> Poursuivre en ajoutant les fichiers dans ce répertoire ? [o]ui  [n]on " ERR1
	echo "    -> Poursuivre en ajoutant les fichiers dans ce répertoire ? [o]ui  [n]on : $ERR1" >> $fichierLog

	if [ "$ERR1" != "o" ] ; then
		echo " "  | tee -a $fichierLog
		echo "Interruption du script." | tee -a $fichierLog
		echo "Aucune action réalisée. Corriger manuellement et relancer le script" | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		setterm -cursor on # Demasquage du curseur
		exit 0
	fi
else
	mkdir "$targetDir/$backupDir"
	if [ "$?" == "0" ]; then # le '$?' == code sortie de la dernière commande'
		echo "OK" | tee -a $fichierLog
	else
		echo "echec" | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		echo "Interruption du script." | tee -a $fichierLog
		echo "Vérifier la cause de la non création du répertoire" | tee -a $fichierLog 
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		echo " " | tee -a $fichierLog
		setterm -cursor on # Demasquage du curseur
		exit 1
	fi
fi




#*******************************************************************************
# Sauvegarde
#*******************************************************************************
echo " " | tee -a $fichierLog
echo " >>>> Début du fonctionnement automatique du script" | tee -a $fichierLog
echo "      Durée d'execution très variables. Vous pouvez faire autre chose " | tee -a $fichierLog
echo "      pendant ce temps" | tee -a $fichierLog

start_time=$(date +%s)
setterm -cursor off # Masquage du curseur

tailleFichierArchive="${tailleFichierArchive}M" # ajout du 'M' pour attribut

if [ "$1" == "test" ] ; then
	test="o"
else
	test="n"
fi

if [ "$test" == "o" ] ; then
	archiverTestSurClefUsb # fonction 'archiver' pour tests avec clef USB
else
	archiver # fonction 'archiver' normal
fi




#*******************************************************************************
# Fin du script
#*******************************************************************************
echo " " | tee -a $fichierLog
echo " " | tee -a $fichierLog
echo "************************************************************************" | tee -a $fichierLog
echo "*                         Fin du script                                *" | tee -a $fichierLog
echo "************************************************************************" | tee -a $fichierLog
finish_time=$(date +%s)
difference=$(( finish_time-start_time ))
he=$(( difference/3600 ))
mi=$(( (( difference/60 ))-$(( he*60 )) ))
se=$(( difference-$(( $(( mi*60 ))+$(( he*3600 )) )) ))
echo "Durée d'execution : $he" heures "$mi" minutes "$se" secondes"" | tee -a $fichierLog

espaceDisponibleDisqueUsbFinale=`df -k $targetDir | grep % | awk '{print $4}'` # '$4' représente le 4ème élement de la ligne 'df -k'
tailleTotalSauvegarde=`expr $espaceDisponibleDisqueUsb - $espaceDisponibleDisqueUsbFinale`

echo -n "Taille de la sauvegarde génerale                     : " | tee -a $fichierLog
afficherTaille $tailleTotalSauvegarde 2 # fonction

echo -n "Espace disponible sur disque USB après la sauvegarde : " | tee -a $fichierLog
afficherTaille $espaceDisponibleDisqueUsbFinale 2 # fonction


# Effacer les fichiers des exclusions des cde 'dar' et 'du' (disk usage)
rm exclusionRepertoiresEtFichiers




#*******************************************************************************
# Sauvegarde des fichiers log et scripts sur le disque USB
#*******************************************************************************
echo " " | tee -a $fichierLog
echo " " | tee -a $fichierLog
echo " " | tee -a $fichierLog
echo -n "Copie fichiers log et scripts sur disque USB... " | tee -a $fichierLog


COPIEOK="o"

cp "$fichierLog"  "$targetDir/$backupDir/"
if [ "$?" -ne "0" ]; then # le '$?' == code sortie de la dernière commande'
	COPIEOK="n"
fi	

cp "SauvegardeGeneralePC.sh"  "$targetDir/$backupDir/"
if [ "$?" -ne "0" ]; then # le '$?' == code sortie de la dernière commande'
	COPIEOK="n"
fi	

cp "SauvegardeGeneralePC_SousProgr1.sh"  "$targetDir/$backupDir/"
if [ "$?" -ne "0" ]; then # le '$?' == code sortie de la dernière commande'
	COPIEOK="n"
fi	

if [ "$COPIEOK" == "o" ]; then
	echo "OK"
	echo " "
	echo " "
	echo " "
else
	echo "echec"
	echo " "
	echo "Interruption du script."
	echo "Réaliser la copie manuellement"
	echo " "
	echo " "
	echo " "
	setterm -cursor on # Demasquage du curseur
	exit 1
fi




setterm -cursor on # Demasquage du curseur
exit 0

